/*

Basic C-Language Application Interface Init/Library for OMEn

by Craig Carmichael, December 1994

(C)1994 Esquimalt Digital Logic Inc.

*/


#include "OMEN268E.H"; /* All are the OMEn system equates (v 2.68) */

/* Some wierd problem with "Laser-C 2.0", or my ignorance of C rules,
causes a WORD to be pushed onto the stack instead of a LONG if the
value pushed will fit into a word. hex 10000 is a "don't care" value
that won't fit. */

#define nul 0x10000

/*	Here's the very first code that's compiled into the file

   Since this replaces the "Init.o" file, we
   (a) relocate the program
   (b) jsr to the main program
   (c) Terminate (_CloseTask) for when (if) "main" exits.

   (This c compiler puts in a zero if dc.w closetask
   is used in advance of defining it, so the OMEn Equates header file,
   which contains no code, just "#defines", is before this.
*/

main();		/* "main" has to be declared before the jsr */

asm {
Ourself:
	lea Ourself(PC),A0		;Point to inside this program
	dc.w relocate			;Call OMEn's DRI++ Relocator 
	dc.w errorreport
	bne.s Quitit
	clr.l -(A7)				;arg C?
	clr.l -(A7)				;arg V?
	jsr main				;Call user's real program
Quitit:
	dc.w closetask			;If he comes back, terminate!
};

/* Typical Atari compiled programs clobber program relocate info with
   the first variable storage space allocations. So, we need to
   define a dummy variable area which we never write to, to protect
   the space. Since we don't know how many bytes of relocation info
   there are at compile time, is would be a real pain to determine
   the exact amount of space to reserve! If the relocate information
   is clobbered, the program will run fine the FIRST time, but if it
   is run again, or copied to another directory subsequent to
   execution, the copies will not work; they will all crash. To find
   out the number of bytes of relocation info:

	1) Get the file size from the directory
    2) Subtract 28 bytes (the GEM/DRI program header)
	2) Open the file with Binary-Ed, in OMEn
	3) Subtract the long word at offset 0x82 from the size
	4) Subtract the long word at offset 0x86 from this result

	The remaining file size is the length of the relocate information.
	The dummy array must cover this number of bytes or more.
	In the first sample program, there were about 20 bytes, so
	int dummy[10] might have covered it. int dummy[4096] will likely
	cover programs well in excess of 200K bytes. (It's kind of a
	waste of 8K of memory for small programs, but convenient.)
*/

int dummy[4096];


/*	The array below contains the returned parameters from any system
	or I/O call. It can be accessed by the call GetReg(Reg#) to
	return any particular register. For each register which can
	return a value from an OMEn system or I/O call, Reg# should be:
		0	D0
		1	D1
		2	D2
		3	D3
		4	D7		(The Error Code Register)
		5	A0
		6	A1
		7	A2
*/
	
typedef struct ResultRegs {
	long D0;
	long D1;
	long D2;
	long D3;
	long D7;
	long A0;
	long A1;
	long A2;
} ResultRegs ;

struct ResultRegs Results;

/*
	These are the "generic" system calls, where the actual call
	opcode is the first parameter supplied. Because these are
	generic, all required register parameters (at least, up to the
	last one actually used) must be supplied by the caller, not just
	the ones detailed for each individual system call. Because "C"
	is case sensitive, all OMEn commands must be entered in lower
	case, since they appear that way in the #defines heder file.
*/

extern int SysCall();

DoSysCall( call, D0, D1, D2, D3, D7, A0, A1, A2 )
int call;
long D0, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
SysCall:
	movem.l D3/D7/A2,-(A7)	; save required registers
	lea		TheCall(PC),A0	; point to opcode to execute
	move.w	16(A7),(A0)		; place the opcode there
	movem.l 18(A7),D0-D3/D7/A0-A2	;fetch the register parameters
TheCall: dc.w	0x4E71		;desired call placed & executed here
	movem.l D0-D3/D7/A0-A2,Results	;save returned parameters
	move.l	D7,D0			;return error code
	movem.l (A7)+,D3/D7/A2	; restore required registers
	rts
 }
}


extern int SCallVerbose();

DoSCallVerbose( call, D0, D1, D2, D3, D4, D5, D6, D7, A0, A1, A2 )
int call;
long D0, D1, D2, D3, D4, D5, D6, D7, A0, A1, A2;
{
 asm {
SCallVerbose:
	movem.l D3/D7/A2,-(A7)	; save required registers
	lea		CallV(PC),A0	; point to opcode to execute
	move.w	16(A7),(A0)		; place the opcode there
	movem.l 18(A7),D0-D7/A0-A2	;fetch the register parameters
CallV: dc.w	0x4E71			;desired call placed & executed here
	movem.l D0-D3/D7/A0-A2,Results	;save returned parameters
	move.l	D7,D0			;return error code
	movem.l (A7)+,D3/D7/A2	; restore required registers
	rts
 }
}


/*
	Below are the OMEn I/O Calls as detailed in Chapter 9 of the
	OMEn Programmers Reference Manual. These are in generic form
	and require the caller to push all the possible I/O command
	register parameters onto the stack, not just the required ones
	for each individual call as per chapter 9.
*/

extern int Post();

DoPost( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
Post:
	movem.l	D3/D7/A2,-(A7)				;save "C" req'd registers
	movem.l	16(A7),D0-D3/D7/A0-A2		;fetch register parameters
	trap	#4		;"_Post"			;make the I/O call
	movem.l	D0-D3/D7/A0-A2,Results		;save returned parameters
	move.l	D7,D0						;return error code
	movem.l	(A7)+,D3/D7/A2				;restore registers
	rts									;& exit to caller
 }
}

extern int Display();

DoDisplay( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
Display:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0-D3/D7/A0-A2
	trap	#5
	movem.l	D0-D3/D7/A0-A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
  }
}


extern int Input();

DoInput( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
Input:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0-D3/D7/A0-A2
	trap 	#6		;"_Input"
	movem.l	D0-D3/D7/A0-A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}

extern int Output();

DoOutput( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
Output:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0/D1/D2/D3/D7/A0/A1/A2
	trap	#7		;"_Output"
	movem.l	D0/D1/D2/D3/D7/A0/A1/A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}

extern int File();

DoFile( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
File:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0/D1/D2/D3/D7/A0/A1/A2
	trap	#8	;"_File"
	movem.l	D0/D1/D2/D3/D7/A0/A1/A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}

extern int MemFile();

DoMemFile( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
MemFile:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0/D1/D2/D3/D7/A0/A1/A2
	trap	#9	;"_MemFile"
	movem.l	D0/D1/D2/D3/D7/A0/A1/A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}

extern int Print();

DoPrint( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
Print:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0-D3/D7/A0-A2
	trap	#10		;"_Print"
	movem.l	D0-D3/D7/A0-A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}

extern int AuxIo();

DoAuxIo( IoCommand, D1, D2, D3, D7, A0, A1, A2 )
long IoCommand, D1, D2, D3, D7, A0, A1, A2;
{
 asm {
AuxIo:
	movem.l	D3/D7/A2,-(A7)
	movem.l	16(A7),D0/D1/D2/D3/D7/A0/A1/A2
	trap	#11	;"_AuxIo"
	movem.l	D0/D1/D2/D3/D7/A0/A1/A2,Results
	move.l	D7,D0
	movem.l	(A7)+,D3/D7/A2
	rts
 }
}


/* End of OMEn I/O Calls */



extern int OpenMem();

DoOpenMem ( D0 )
long D0;
{
 asm {
OpenMem:
	move.l	D7,-(A7)
	move.l 8(A7),D0			; Size of memory to open
	dc.w openmem			; OMEN memory allocation call
	move.l D7,D0			; return the error code
	move.l A0,Results.A0	; Handle to opened memory
	move.l	(A7)+,D7
	rts
 }
}

int WaitMail()
{
 asm {
	movem.l	D3/D7,-(A7)
	moveq	#waitinput,D0
	dc.w	post
	movem.l	D0/D1/D2/D3,Results
	movem.l	(A7)+,D3/D7
	}
}

extern int GetReg();
DoGetReg( regNum )
int regNum;
{
 asm {
GetReg:
	move.w	4(A7),D0
	lsl.w	#2,D0
	lea		Results,A0
	move.l	0(A0,D0.w),D0
	rts
 }
}


PrintStringEOT ( A0 )
long A0;
{
 asm {
	move.l	D7,-(A7)
	move.l	12(A7),A0		; get address of text string
	dc.w	prstreot		; do the OMEN line-A system call
	move.l	A0,Results.A0	; save EOT address
	move.l	D7,D0			; return error code
	move.l	(A7)+,D7
 }
}


CloseTask()
{
asm {
	dc.w	closetask
	};
}

